@@ -28,7 +28,7 @@
 #include "bufferadaptor.h"
 
 #define SINK_ENTITY(TYPE, LOWERCASENAME) \
-    static constexpr const char *name = #LOWERCASENAME; \
+    inline static constexpr const char *name = #LOWERCASENAME; \
     typedef QSharedPointer<TYPE> Ptr; \
     using Entity::Entity; \
     TYPE() = default; \
@@ -39,7 +39,7 @@
 
 #define SINK_PROPERTY(TYPE, NAME, LOWERCASENAME) \
     struct NAME { \
-        static constexpr const char *name = #LOWERCASENAME; \
+        SINK_EXPORT inline static constexpr const char *name = #LOWERCASENAME; \
         typedef TYPE Type; \
     }; \
     void set##NAME(const TYPE &value) { setProperty(NAME::name, QVariant::fromValue(value)); } \
@@ -48,7 +48,7 @@
 
 #define SINK_EXTRACTED_PROPERTY(TYPE, NAME, LOWERCASENAME) \
     struct NAME { \
-        static constexpr const char *name = #LOWERCASENAME; \
+        SINK_EXPORT inline static constexpr const char *name = #LOWERCASENAME; \
         typedef TYPE Type; \
     }; \
     void setExtracted##NAME(const TYPE &value) { setProperty(NAME::name, QVariant::fromValue(value)); } \
@@ -56,7 +56,7 @@
 
 #define SINK_STATUS_PROPERTY(TYPE, NAME, LOWERCASENAME) \
     struct NAME { \
-        static constexpr const char *name = #LOWERCASENAME; \
+        SINK_EXPORT inline static constexpr const char *name = #LOWERCASENAME; \
         typedef TYPE Type; \
     }; \
     void setStatus##NAME(const TYPE &value) { setProperty(NAME::name, QVariant::fromValue(value)); } \
@@ -64,7 +64,7 @@
 
 #define SINK_REFERENCE_PROPERTY(TYPE, NAME, LOWERCASENAME) \
     struct NAME { \
-        static constexpr const char *name = #LOWERCASENAME; \
+        SINK_EXPORT inline static constexpr const char *name = #LOWERCASENAME; \
         typedef Reference Type; \
         typedef ApplicationDomain::TYPE ReferenceType; \
     }; \
@@ -74,7 +74,7 @@
 
 #define SINK_INDEX_PROPERTY(TYPE, NAME, LOWERCASENAME) \
     struct NAME { \
-        static constexpr const char *name = #LOWERCASENAME; \
+        SINK_EXPORT inline static constexpr const char *name = #LOWERCASENAME; \
         typedef TYPE Type; \
     }; \
 
@@ -82,24 +82,24 @@
 namespace Sink {
 namespace ApplicationDomain {
 
-enum SINK_EXPORT ErrorCode {
+enum ErrorCode {
     NoError = 0,
     UnknownError,
-    NoServerError,
-    ConnectionError,
-    LoginError,
-    ConfigurationError,
-    TransmissionError,
-    ConnectionLostError,
-    MissingCredentialsError,
+    NoServerError, //Failed to find the server. E.g. due to failed hostname resolution.
+    ConnectionError, //The connection somehow failed (but the server was found)
+    LoginError, //Login failed (but we managed to connect) -> Typically wrong credentials.
+    ConfigurationError, //The configuration is broken.
+    TransmissionError, //There was an error while transmission (e.g. while uploading something)
+    ConnectionLostError, //We lost the connection to the server.
+    MissingCredentialsError, //There are no credentials available, but required.
     ResourceCrashedError
 };
 
-enum SINK_EXPORT SuccessCode {
+enum SuccessCode {
     TransmissionSuccess
 };
 
-enum SINK_EXPORT SyncStatus {
+enum SyncStatus {
     NoSyncStatus,
     SyncInProgress,
     SyncError,
@@ -117,7 +117,7 @@ enum SINK_EXPORT SyncStatus {
  * * If an error occurred that keeps the resource from operating (so non transient), the resource enters the error state.
  * * If a long running operation is started the resource goes to the busy state (and return to the previous state after that).
  */
-enum SINK_EXPORT Status {
+enum Status {
     NoStatus,
     OfflineStatus,
     ConnectedStatus,
@@ -292,7 +292,7 @@ public:
     int count() const;
 
 private:
-    friend QDebug operator<<(QDebug, const ApplicationDomainType &);
+    friend SINK_EXPORT QDebug operator<<(QDebug, const ApplicationDomainType &);
     QSharedPointer<BufferAdaptor> mAdaptor;
     QSharedPointer<QSet<QByteArray>> mChangeSet;
     /*
@@ -319,7 +319,7 @@ SINK_EXPORT QDebug operator<< (QDebug d, const Reference &ref);
 
 
 struct SINK_EXPORT SinkAccount : public ApplicationDomainType {
-    static constexpr const char *name = "account";
+    inline static constexpr const char *name = "account";
     typedef QSharedPointer<SinkAccount> Ptr;
     explicit SinkAccount(const QByteArray &resourceInstanceIdentifier, const QByteArray &identifier, qint64 revision, const QSharedPointer<BufferAdaptor> &adaptor);
     explicit SinkAccount(const QByteArray &identifier);
@@ -340,7 +340,7 @@ struct SINK_EXPORT SinkAccount : public ApplicationDomainType {
  * and for creating and removing resource instances.
  */
 struct SINK_EXPORT SinkResource : public ApplicationDomainType {
-    static constexpr const char *name = "resource";
+    inline static constexpr const char *name = "resource";
     typedef QSharedPointer<SinkResource> Ptr;
     explicit SinkResource(const QByteArray &resourceInstanceIdentifier, const QByteArray &identifier, qint64 revision, const QSharedPointer<BufferAdaptor> &adaptor);
     explicit SinkResource(const QByteArray &identifier);
@@ -351,6 +351,7 @@ struct SINK_EXPORT SinkResource : public ApplicationDomainType {
     SINK_PROPERTY(QByteArray, ResourceType, type);
     SINK_PROPERTY(QByteArrayList, Capabilities, capabilities);
     SINK_STATUS_PROPERTY(int, Status, status);
+    SINK_PROPERTY(QString, Server, server);
 };
 
 struct SINK_EXPORT Entity : public ApplicationDomainType {
@@ -366,6 +367,7 @@ struct SINK_EXPORT Addressbook : public Entity {
     SINK_REFERENCE_PROPERTY(Addressbook, Parent, parent);
     SINK_PROPERTY(QString, Name, name);
     SINK_EXTRACTED_PROPERTY(QDateTime, LastUpdated, lastUpdated);
+    SINK_PROPERTY(bool, Enabled, enabled);
 };
 
 struct SINK_EXPORT Contact : public Entity {
@@ -389,20 +391,42 @@ struct SINK_EXPORT Contact : public Entity {
     SINK_REFERENCE_PROPERTY(Addressbook, Addressbook, addressbook);
 };
 
+SINK_EXPORT QDebug operator<< (QDebug d, const Contact::Email &);
+
+struct SINK_EXPORT Calendar : public Entity {
+    SINK_ENTITY(Calendar, calendar);
+    SINK_PROPERTY(QString, Name, name);
+    SINK_PROPERTY(QByteArray, Color, color);
+    SINK_PROPERTY(bool, Enabled, enabled);
+    SINK_PROPERTY(QByteArrayList, ContentTypes, contentTypes);
+};
+
 struct SINK_EXPORT Event : public Entity {
     SINK_ENTITY(Event, event);
-    SINK_PROPERTY(QString, Uid, uid);
-    SINK_PROPERTY(QString, Summary, summary);
-    SINK_PROPERTY(QString, Description, description);
-    SINK_PROPERTY(QByteArray, Attachment, attachment);
+    SINK_EXTRACTED_PROPERTY(QString, Uid, uid);
+    SINK_EXTRACTED_PROPERTY(QString, Summary, summary);
+    SINK_EXTRACTED_PROPERTY(QString, Description, description);
+    SINK_EXTRACTED_PROPERTY(QDateTime, StartTime, startTime);
+    SINK_EXTRACTED_PROPERTY(QDateTime, EndTime, endTime);
+    SINK_EXTRACTED_PROPERTY(bool, AllDay, allDay);
+    SINK_EXTRACTED_PROPERTY(bool, Recurring, recurring);
+    SINK_PROPERTY(QByteArray, Ical, ical);
+    SINK_REFERENCE_PROPERTY(Calendar, Calendar, calendar);
 };
 
 struct SINK_EXPORT Todo : public Entity {
     SINK_ENTITY(Todo, todo);
-};
-
-struct SINK_EXPORT Calendar : public Entity {
-    SINK_ENTITY(Calendar, calendar);
+    SINK_EXTRACTED_PROPERTY(QString, Uid, uid);
+    SINK_EXTRACTED_PROPERTY(QString, Summary, summary);
+    SINK_EXTRACTED_PROPERTY(QString, Description, description);
+    SINK_EXTRACTED_PROPERTY(QDateTime, CompletedDate, completedDate);
+    SINK_EXTRACTED_PROPERTY(QDateTime, DueDate, dueDate);
+    SINK_EXTRACTED_PROPERTY(QDateTime, StartDate, startDate);
+    SINK_EXTRACTED_PROPERTY(QString, Status, status);
+    SINK_EXTRACTED_PROPERTY(int, Priority, priority);
+    SINK_EXTRACTED_PROPERTY(QStringList, Categories, categories);
+    SINK_PROPERTY(QByteArray, Ical, ical);
+    SINK_REFERENCE_PROPERTY(Calendar, Calendar, calendar);
 };
 
 struct SINK_EXPORT Folder : public Entity {
@@ -439,7 +463,7 @@ struct SINK_EXPORT Mail : public Entity {
     SINK_PROPERTY(bool, Trash, trash);
     SINK_PROPERTY(bool, Sent, sent);
     SINK_EXTRACTED_PROPERTY(QByteArray, MessageId, messageId);
-    SINK_EXTRACTED_PROPERTY(QByteArray, ParentMessageId, parentMessageId);
+    SINK_EXTRACTED_PROPERTY(QByteArrayList, ParentMessageIds, parentMessageIds);
     SINK_INDEX_PROPERTY(QByteArray, ThreadId, threadId);
 };
 
@@ -477,6 +501,10 @@ struct SINK_EXPORT CardDavResource {
     static SinkResource create(const QByteArray &account);
 };
 
+struct SINK_EXPORT CalDavResource {
+    static SinkResource create(const QByteArray &account);
+};
+
 namespace ResourceCapabilities {
 namespace Mail {
     static constexpr const char *mail = "mail";
@@ -493,6 +521,16 @@ namespace Contact {
     static constexpr const char *addressbook = "addressbook";
     static constexpr const char *storage = "contact.storage";
 };
+namespace Event {
+    static constexpr const char *event = "event";
+    static constexpr const char *calendar = "calendar";
+    static constexpr const char *storage = "event.storage";
+};
+namespace Todo {
+    static constexpr const char *todo = "todo";
+    static constexpr const char *calendar = "calendar";
+    static constexpr const char *storage = "todo.storage";
+};
 };
 
 namespace SpecialPurpose {
@@ -510,7 +548,7 @@ namespace Mail {
  * Do not store these types to disk, they may change over time.
  */
 template<class DomainType>
-QByteArray SINK_EXPORT getTypeName()
+QByteArray getTypeName()
 {
     return DomainType::name;
 }
@@ -522,7 +560,7 @@ bool SINK_EXPORT isGlobalType(const QByteArray &type);
 
 /**
  * Type implementation.
- * 
+ *
  * Needs to be implemented for every application domain type.
  * Contains all non-resource specific, but type-specific code.
  */
@@ -545,6 +583,8 @@ class SINK_EXPORT TypeImplementation;
     REGISTER_TYPE(Sink::ApplicationDomain::Contact) \
     REGISTER_TYPE(Sink::ApplicationDomain::Addressbook) \
     REGISTER_TYPE(Sink::ApplicationDomain::Event) \
+    REGISTER_TYPE(Sink::ApplicationDomain::Todo) \
+    REGISTER_TYPE(Sink::ApplicationDomain::Calendar) \
     REGISTER_TYPE(Sink::ApplicationDomain::Mail) \
     REGISTER_TYPE(Sink::ApplicationDomain::Folder) \
     REGISTER_TYPE(Sink::ApplicationDomain::SinkResource) \
@@ -560,6 +600,36 @@ SINK_EXPORT QDataStream &operator>>(QDataStream &in, Sink::ApplicationDomain::Re
 SINK_REGISTER_TYPES()
 #undef REGISTER_TYPE
 
+
+template <template<typename> class Func>
+struct TypeHelper {
+    const QByteArray type;
+
+    template <typename R, typename ...Args>
+    R operator()(Args && ... args) const {
+        if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Folder>()) {
+            return Func<Sink::ApplicationDomain::Folder>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Mail>()) {
+            return Func<Sink::ApplicationDomain::Mail>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Event>()) {
+            return Func<Sink::ApplicationDomain::Event>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Todo>()) {
+            return Func<Sink::ApplicationDomain::Todo>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Calendar>()) {
+            return Func<Sink::ApplicationDomain::Calendar>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Contact>()) {
+            return Func<Sink::ApplicationDomain::Contact>{}(std::forward<Args>(args)...);
+        } else if (type == Sink::ApplicationDomain::getTypeName<Sink::ApplicationDomain::Addressbook>()) {
+            return Func<Sink::ApplicationDomain::Addressbook>{}(std::forward<Args>(args)...);
+        } else {
+            Q_ASSERT(false);
+        }
+        //Silence compiler warning
+        return Func<Sink::ApplicationDomain::Mail>{}(std::forward<Args>(args)...);
+    }
+};
+
+
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::ApplicationDomainType)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::ApplicationDomainType::Ptr)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Entity)
